#ifndef __command
#define __command

#include <vector>
#include <map>
#include <string>

#include "help.h"

struct tokenizer;

using namespace std;

struct command {
	
	string longname, shortname;
	help f1;
	
	command (const string& ln, const string& sn, const string& fname) : longname (ln), shortname (sn), f1(fname) {}
	bool operator== (const command& c) {
		return ((c.longname == longname) || (c.shortname == shortname));
	}
	virtual bool operator() (tokenizer& tz) { return true; }
	virtual ~command () {}
	
};

struct cmdlist {
	
	vector<command*> cmds;
	int ncmds;
	
	string result; // holds last run command output
	char buf [1024]; // for console printing
	
	void add (command* cmd);
	command* run (const string& cmd);
	bool f1 (const string& cmd);
	void list ();
		
};

struct din;

struct setv : command { // set var
	din* d;
	public:
		setv (din* dd, const string& ln, const string& sn, const string& hlp) : command (ln, sn, hlp), d(dd) {}
		bool operator() (tokenizer& tz);
		~setv () {}
};

struct getv : command { // get var
	din* d;
	public:
		getv (din* dd, const string& ln, const string& sn, const string& hlp) : command (ln, sn, hlp), d(dd) {}
		bool operator() (tokenizer& tz);
		~getv () {}
};

struct delay;

struct set_delay: command {
	delay* left;
	delay* right;
	public:
		set_delay (delay* l, delay* r, const string& ln, const string& sn, const string& hlp) : command (ln, sn, hlp), left(l), right(r) {}
		bool operator() (tokenizer& tz);
		~set_delay () {}
};

struct get_delay: command {
	delay* left;
	delay* right;
	public:
		get_delay (delay* l, delay* r, const string& ln, const string& sn, const string& hlp) : command (ln, sn, hlp), left(l), right(r) {}
		bool operator() (tokenizer& tz);
		~get_delay () {}
};

struct beat2value;

struct bpm_com {
	static const int NUM = 5;
	static const char* str [NUM];
	static beat2value* bv [NUM];
	int set [NUM];
	void fill (const string& name);
	void setset (int v) { for (int i = 0; i < NUM; ++i) set[i]=v;}
};

struct set_bpm: command, bpm_com {
	set_bpm (const string& ln, const string& sn, const string& hlp) : command (ln, sn, hlp) {}
	bool operator() (tokenizer& tz);
	~set_bpm () {}
};

struct get_bpm: command, bpm_com {
	get_bpm (const string& ln, const string& sn, const string& hlp) : command (ln, sn, hlp) {}
	bool operator() (tokenizer& tz);
	~get_bpm () {}
};

struct set_style : command, bpm_com {
	set_style (const string& ln, const string& sn, const string& hlp) : command (ln, sn, hlp) {}
	bool operator() (tokenizer& tz);
	~set_style () {}
};

struct get_style : command, bpm_com {
	get_style (const string& ln, const string& sn, const string& hlp) : command (ln, sn, hlp) {}
	bool operator() (tokenizer& tz);
	~get_style () {}
};

struct gaters : command {
	din* d;
	gaters (din* dd, const string& ln, const string& sn, const string& hlp) : command (ln, sn, hlp), d(dd) {}
	bool operator() (tokenizer& tz);
	void set_state (const string& target, const bool& state);
	string get_state (const string& target);
	~gaters () {}
};

struct add_scale: command {
	public:
		add_scale (const string& ln, const string& sn, const string& hlp) : command (ln, sn, hlp) {}
		bool operator () (tokenizer& tz);
		~add_scale () {}
};

struct ls_scale: command {
	public:
		ls_scale (const string& ln, const string& sn, const string& hlp) : command (ln, sn, hlp) {}
		bool operator () (tokenizer& tz);
		~ls_scale () {}
};

struct remove_scale : command {
	public:
		remove_scale (const string& ln, const string& sn, const string& hlp) : command (ln, sn, hlp) {}
		bool operator () (tokenizer& tz);
		~remove_scale () {}
};

struct load_scale : command {
	din* d;
	public:
		load_scale (din* dd, const string& ln, const string& sn, const string& hlp) : command (ln, sn, hlp), d(dd) {}
		bool operator () (tokenizer& tz);
		~load_scale () {}
};

struct show_help : command {
	cmdlist* lst;
	public:
		show_help (const string& ln, const string& sn, const string& hlp, cmdlist* lt) : command (ln, sn, hlp), lst (lt) {}
		bool operator() (tokenizer& tz);
		~show_help () {}
};

struct key : command {
	din* d;
	public:
		key (din* dd, const string& ln, const string& sn, const string& hlp) : command (ln, sn, hlp), d(dd) {}
		bool operator() (tokenizer& tz);
		~key () {}
		void find_nearest_note (string& note, float& frequency, float& dist);
		bool check (float f);
};

struct note_distance: command {
	public:
		note_distance (const string& ln, const string& sn, const string& hlp) : command (ln, sn, hlp) {}
		bool operator() (tokenizer& tz);
		~note_distance () {}
};

struct chord: command {
	public:
		chord (const string& ln, const string& sn, const string& hlp) : command (ln, sn, hlp) {}
		bool operator() (tokenizer& tz);
		~chord () {}
};

struct set_kern: command {
	
	public:
		set_kern (const string& ln, const string& sn, const string& hlp) : command (ln, sn, hlp) {}
		bool operator() (tokenizer& tz);
		~set_kern () {}
	
};

struct get_kern: command {
	
	public:
		get_kern (const string& ln, const string& sn, const string& hlp) : command (ln, sn, hlp) {}
		bool operator() (tokenizer& tz);
		~get_kern () {}
	
};

struct set_font_size: command {
	
	public:
		set_font_size (const string& ln, const string& sn, const string& hlp) : command (ln, sn, hlp) {}
		bool operator() (tokenizer& tz);
		~set_font_size () {}
	
};

struct get_font_size: command {
	
	public:
		get_font_size (const string& ln, const string& sn, const string& hlp) : command (ln, sn, hlp) {}
		bool operator() (tokenizer& tz);
		~get_font_size () {}
	
};

struct load_resonators : command {
	din* d;
	load_resonators (din* dd, const string& ln, const string& sn, const string& hlp) : command (ln, sn, hlp), d(dd) {}
	bool operator() (tokenizer& tz);
	~load_resonators () {}
};

struct save_resonators : command {
	din* d;
	save_resonators (din* dd, const string& ln, const string& sn, const string& hlp) : command (ln, sn, hlp), d(dd) {}
	bool operator() (tokenizer& tz);
	~save_resonators () {}
};

struct delete_disabled_resonators : command {
	din* d;
	delete_disabled_resonators (din* dd, const string& ln, const string& sn, const string& hlp) : command (ln, sn, hlp), d(dd) {}
	bool operator() (tokenizer& tz);
	~delete_disabled_resonators () {}
};

struct din_info;

struct notation : command {

	din* d;
	notation (din* dd, const string& ln, const string& sn, const string& hlp) : command (ln, sn, hlp), d(dd) {}
	bool operator() (tokenizer& tz);
	~notation () {}

};

struct echo : command {
	echo (const string& ln, const string& sn, const string& hlp) : command (ln, sn, hlp) {}
	bool operator() (tokenizer& tz);
	~echo () {}
};

struct set_pitch_and_volume : command {
	
	din* d;
	
	set_pitch_and_volume (din* dd, const string& ln, const string& sn, const string& hlp) : command (ln, sn, hlp), d(dd) {}
	bool operator () (tokenizer& tz);
	~set_pitch_and_volume () {}
	
};

struct mouse : command {
	mouse (const string& ln, const string& sn, const string& hlp) : command (ln, sn, hlp) {}
	bool operator() (tokenizer& tz);
	~mouse () {}
};

struct curve_name : command { // change picked curve name
	curve_name (const string& ln, const string& sn, const string& hlp) : command (ln, sn, hlp) {}
	bool operator() (tokenizer& tz);
	~curve_name () {}
};

struct curve_value : command { // change picked curve value 
	curve_value (const string& ln, const string& sn, const string& hlp) : command (ln, sn, hlp) {}
	bool operator() (tokenizer& tz);
	void parse (const string& sx, const string& sy, float& x, float& y);
	~curve_value () {}
};

struct curve__library : command { // manipulate curve library attached to current curve editor
	curve__library (const string& ln, const string& sn, const string& hlp) : command (ln, sn, hlp) {}
	bool operator() (tokenizer& tz);
	~curve__library () {}
};

struct quit : command {
	quit (const string& ln, const string& sn, const string& hlp) : command (ln, sn, hlp) {}
	bool operator() (tokenizer& tz);
	~quit () {}
};

struct set_curve_editor : command {
	set_curve_editor (const string& ln, const string& sn, const string& hlp) : command (ln, sn, hlp) {}
	bool operator() (tokenizer& tz);
	~set_curve_editor () {}
};

struct get_curve_editor : command {
	get_curve_editor (const string& ln, const string& sn, const string& hlp) : command (ln, sn, hlp) {}
	bool operator() (tokenizer& tz);
	~get_curve_editor () {}
};

#endif
