#ifndef _CONSOLE
#define _CONSOLE

#define eol console::EOL
#define cll console::CLL
#define tab "  "

#include "box.h"
#include "ui.h"
#include "tokenizer.h"

#include <SDL/SDL_opengl.h>
#include <vector>
#include <string>
#include <sstream>

struct decoration {
  float r, g, b; // color 
  int lw; // line width
  decoration (float rr = 0.5, float gg = 0.5, float bb = 0.5, int w = 1);
};

struct mesg { // console holds a vector of mesgs
  string text; // text
  decoration dec; // decoration
  mesg (string t, decoration d) : text(t), dec (d) {}
  mesg () {}
};

struct console : ui {

	static const char EOL = '\n'; // end of last mesg
	static const char CLL = '\b'; // clear last mesg

  vector<mesg> lines;

  int nlines;
  int lines_per_screen;
  int startl;

  tokenizer tz;
  mesg last_line;

	vector<string> history;
	int hid;

	console ();
	console& operator<< (unsigned int i);
	console& operator<< (unsigned long i);
	console& operator<< (unsigned long long i);
	console& operator<< (int i);
	console& operator<< (const string& s);
	console& operator<< (float f);
	console& operator<< (double d);
	console& operator<< (char c);
	console& operator<< (const decoration& d);

  void up (int i);
  void down (int i);
  void last (); // ensures last mesg is always displayed

  void add_last_line ();
  void push_back (const mesg& ln);
	void clear_last_line ();
	
	void del ();
	void clear ();
	void pgdn ();
	void pgup ();
	void home ();
	void end ();
	
	static const decoration light;
	static const decoration yellow;
	static const decoration green;
	static const decoration red;
	static const decoration cyan;
	static const decoration bold;
  box<int> win;
  int startx, starty;
  int char_width, line_height;

	void set_window (const box<int>& w);
	void calc_draw_params ();
	void draw ();
	
	bool handle_input ();
	
	int command_mode;
	string command;
	
	bool rollup;
	
	string ps1;
		
	int loc;
	int locx;
		
	~console();
    
};

template <class T> inline console& operator<< (console& c, const box<T>& b) {
	c << b.left << ' ' << b.bottom << ' ' << b.right << ' ' << b.top;
	return c;
}

extern console cons;
#endif
