#ifndef _FONT
#define _FONT

#include <map>
#include <string>
#include <fstream>
#include <SDL/SDL_opengl.h>
#include <glyph.h>
using namespace std;

struct font {
   
	string fname; 
	string name;

	int nchars;
	int max_char_width, max_char_height;

	int xsize, ysize; // x and y size of cell

	int charspc, wordspc;

	int headroom;
		
	map <char, glyph> characters; // vector desc of chars

	map < char, map<char, int> > kern; // char-char kerning
	 
	int modified; // if modified will save
 
	font (const string& fn);
	~font ();
	void load (const string& fn, int load_cell_size = 1);
	void load (ifstream& file, int load_cell_size);
	void save ();
	void cell_size (int xs, int ys);
	
	int char_width (char c);
	int char_height (char c);
	
	void draw_char (char c, int x, int y, int z = 0);
	const string& filename () const { return fname;}
	
	const map<char, glyph>& get_chars ();
	void set_chars (const map<char, glyph>& chars);
		
};

extern font fnt;

inline font& get_font () {
	return fnt;
}

inline int get_line_height () {
  return (fnt.max_char_height + fnt.headroom);
}

inline int max_char_width () {
  return fnt.max_char_width;
}

inline void draw_char (char c, int x, int y, int z = 0) {
  font& f = get_font ();
  f.draw_char (c, x, y, z);
}

int draw_string (const string& s, int x, int y, int z = 0);
float x_after_string (const string& s);

#endif
