#ifndef _MULTICURVE
#define _MULTICURVE

#include <string>
#include <fstream>
#include <vector>
#include "point.h"
#include "curve.h"
#include <iostream>
using namespace std;

typedef vector< point<float> > points_vector;

struct multi_curve {
	
	string name; // name

	float r, g, b; // color
	
	// curve profile points used for drawing & solving
	static const int MAX_POINTS = 2048;
	crvpt points [MAX_POINTS];
	int npoints;
  
  // curve info	
	points_vector vertices;
	points_vector left_tangents;
	points_vector right_tangents;
	
	vector<curve> curv; // bezier curves that makes up this multi_curve.
	vector<int> eval; // eval status of each bezier curve
	
	float resolution; // resolution of all bezier curves
	
	multi_curve (const string& filename);
	multi_curve ();
	multi_curve (const multi_curve& src);
	multi_curve& operator= (const multi_curve& src);
	void copy (const multi_curve& src);
	
	void clear (int all = 1);
	
	int num_vertices ();

	void add_vertex (float x, float y);
	void add_left_tangent (float x, float y);
	void add_right_tangent (float x, float y);

	bool set_vertex (int i, float x, float y);
	bool set_left_tangent (int i, float x, float y);
	bool set_right_tangent (int i, float x, float y);
	
	bool insert (float x, float y, float tx, float ty);
	bool remove (int i);

	void get_vertex (int i, float& x, float& y);
	void get_left_tangent (int i, float& x, float& y);
	void get_right_tangent (int i, float& x, float& y);

	void set_resolution (float d);
	float get_resolution ();

	void set_color (); // sets random color
	void set_color (float rr, float gg, float bb);
	void get_color (float& rr, float& gg, float& bb) { rr = r; gg = g; bb = b;}
	void get_name_and_color (string& n, float& rr, float& gg, float& bb) { n = name; rr = r; gg = g; bb = b;}
	void set_name_and_color (const string& n, float rr, float gg, float bb) { name = n; r = rr; g = gg; b = bb;}

	void force_eval ();
	void evaluate ();

	void load (const string& filename);
	void load (ifstream& file);
	void save (const string& filename);
	void save (ofstream& file);
	crvpt* get_points () {return points;}
	int num_points () {return npoints;}
		
};

void create_polyline (multi_curve& crv, const points_vector& pts);
void convert2_catmull_rom (multi_curve& crv, float tangent_size);
void convert2_polyline (multi_curve& crv);

#endif
