#include "audio.h"
#include "chrono.h"
#include <math.h>
#include <fstream>
#include <stdlib.h>
using namespace std;

extern int SAMPLE_RATE;


static const char *name = "din", *L = "L", * R = "R", *dinL = "din:L", *dinR = "din:R";

audio_out::audio_out () {
	open ();
}

audio_out::~audio_out () {
	stop ();
	close ();
}

int audio_out::open() {

	jack_status_t status; 
	jack_options_t options = JackNullOption;
	
	client = jack_client_open (name, options, &status, 0);
	if (client == 0) {
		cout << "couldnt connect to jack server." << endl;
		return 1;
	}
	
	sample_rate = jack_get_sample_rate (client);
	extern int SAMPLE_RATE; SAMPLE_RATE = (int) sample_rate;
	jack_set_sample_rate_callback (client, sample_rate_changed, this);
	
	sample_buffer_size = sizeof (sample_t) * jack_get_buffer_size (client);
	jack_set_buffer_size_callback (client, buffer_size_changed, this);
	
	midi_in = jack_port_register (client, "in", JACK_DEFAULT_MIDI_TYPE, JackPortIsInput, 0); 
	out_left = jack_port_register (client, L, JACK_DEFAULT_AUDIO_TYPE, JackPortIsOutput, 0);
	out_right = jack_port_register (client, R, JACK_DEFAULT_AUDIO_TYPE, JackPortIsOutput, 0);
	if ((out_left == 0) && (out_right == 0)) {
		cout << "couldnt create output ports" << endl;
		return 1;
	}
	
	lcons = rcons = 0;
	
	return 0;
			
}

int audio_out::close () {
	if (client) jack_client_close (client);
	return 0;
}

int audio_out::start () {
	if (client) {
		
		jack_set_process_callback (client, jack_audio_wanted, this);
		jack_activate (client);

		if (lcons) {
			for (int i = 0; i < lcons; ++i) jack_connect (client, dinL, con_left[i]);
			free (con_left);
		} else jack_connect (client, dinL, "system:playback_1");
		
		if (rcons) {
			for (int i = 0; i < lcons; ++i) jack_connect (client, dinR, con_right[i]);
			free (con_right);
		} else jack_connect (client, dinR, "system:playback_2");
	
	}
	return 0;
}

int audio_out::stop () {  
	if (client) {
		lcons = jack_port_connected (out_left);
		rcons = jack_port_connected (out_right);
		if (lcons) con_left = jack_port_get_connections (out_left);
		if (rcons) con_right = jack_port_get_connections (out_right);
		jack_deactivate (client);
	}
	return 0;
}
