#include "tokenizer.h"

#include "command.h"

#include <iostream>
#include "tcl_interp.h"

extern cmdlist cmdlst;

using namespace std;

int tcl_run (ClientData cd, Tcl_Interp* ti, int objc, Tcl_Obj* CONST objv[]) {
	string cmd, spc (" ");
	int n = objc - 1;
	for (int i = 0; i < n; ++i) cmd = cmd + Tcl_GetString (objv[i]) + spc;
	cmd += Tcl_GetString (objv[n]);
	cmdlst.run (cmd); Tcl_SetResult (ti, (char *) cmdlst.result.c_str(), TCL_VOLATILE); 
	return 0;
}

#define DEFINE_TCL_CMD(X) inline int (X) (ClientData cd, Tcl_Interp* ti, int objc, Tcl_Obj* CONST objv[]) { return tcl_run (cd, ti, objc, objv);}

DEFINE_TCL_CMD(tcl_tonic)
DEFINE_TCL_CMD(tcl_setv)
DEFINE_TCL_CMD(tcl_getv)
DEFINE_TCL_CMD(tcl_set_delay)
DEFINE_TCL_CMD(tcl_get_delay)
DEFINE_TCL_CMD(tcl_set_bpm)
DEFINE_TCL_CMD(tcl_get_bpm)
DEFINE_TCL_CMD(tcl_set_style)
DEFINE_TCL_CMD(tcl_get_style)
DEFINE_TCL_CMD(tcl_gaters)
DEFINE_TCL_CMD(tcl_add_scale)
DEFINE_TCL_CMD(tcl_ls_scale)
DEFINE_TCL_CMD(tcl_remove_scale)
DEFINE_TCL_CMD(tcl_load_scale)
DEFINE_TCL_CMD(tcl_set_kern)
DEFINE_TCL_CMD(tcl_get_kern)
DEFINE_TCL_CMD(tcl_set_font_size)
DEFINE_TCL_CMD(tcl_get_font_size)
DEFINE_TCL_CMD(tcl_load_resonators)
DEFINE_TCL_CMD(tcl_save_resonators)
DEFINE_TCL_CMD(tcl_delete_disabled_resonators)
DEFINE_TCL_CMD(tcl_note_distance)
DEFINE_TCL_CMD(tcl_chord)
DEFINE_TCL_CMD(tcl_notation)
DEFINE_TCL_CMD(tcl_echo)
DEFINE_TCL_CMD(tcl_curve_value)
DEFINE_TCL_CMD(tcl_curve_name)
DEFINE_TCL_CMD(tcl_curve_library)
DEFINE_TCL_CMD(tcl_morse_code)
DEFINE_TCL_CMD(tcl_bot)
DEFINE_TCL_CMD(tcl_help);
DEFINE_TCL_CMD(tcl_set_console);
DEFINE_TCL_CMD(tcl_audio_out);
DEFINE_TCL_CMD (tcl_quit);
DEFINE_TCL_CMD (tcl_set_editor);
DEFINE_TCL_CMD (tcl_get_editor);


tcl_interp::tcl_interp () {
	
	Tcl_FindExecutable (0);
	
	interp = Tcl_CreateInterp ();
	
	if (interp) {
		
		cout << "created Tcl interpreter" << endl;
		
		Tcl_Init (interp);
		
		// ++ din commands
		//
		int ncmds = 35;
		tclcmd cmd_funcs [] = {
			tcl_tonic, tcl_setv, tcl_getv, tcl_set_delay, tcl_get_delay, tcl_set_bpm, tcl_get_bpm, tcl_set_style, tcl_get_style,
			tcl_gaters, tcl_add_scale, tcl_ls_scale, tcl_remove_scale, tcl_load_scale, tcl_set_kern, tcl_get_kern, tcl_set_font_size,
			tcl_get_font_size, tcl_load_resonators, tcl_save_resonators, tcl_delete_disabled_resonators, tcl_note_distance,
			tcl_chord, tcl_notation, tcl_echo, tcl_curve_value, tcl_curve_name, tcl_curve_library, tcl_morse_code, tcl_bot,
			tcl_help, tcl_audio_out, tcl_quit, tcl_set_editor, tcl_get_editor
		};
														
		const char* cmd_long [] = {
			"tonic", "setv", "getv", "set_delay", "get_delay", "set_bpm", "get_bpm", "set_style", "get_style",
			"gaters", "add_scale", "ls_scale", "remove_scale", "load_scale", "set_kern", "get_kern", "set_font_size",
			"get_font_size", "load_resonators", "save_resonators", "delete_disabled_resonators", "note_distance",
			"chord", "notation", "echo", "curve_value", "curve_name", "curve_library", "morse_code", "bot", "help"
			,"aout", "quit", "set_curve_editor", "get_curve_editor"
		};
		
		const char* cmd_short [] = {
			"key", "sv", "gv", "sd", "gd", "sb", "gb", "ss", "gs",
			"g", "ar", "ls", "rs", "los", "sk", "gk", "sfs",
			"gfs", "lre", "sre", "ddr", "nd", "ch", "no", "echo", "cv", "cn", "cl", "mc", 
			":", "?", "ao", "exit", "sced", "gced"
		};
		
		for (int i = 0; i < ncmds; ++i) {
			Tcl_CreateObjCommand (interp, cmd_long[i], cmd_funcs[i], 0, 0);
			Tcl_CreateObjCommand (interp, cmd_short[i], cmd_funcs[i], 0, 0);
		}
		
		cout << "+++ added " << ncmds << " din commands to the Tcl interpreter +++" << endl;
		
	} else cout << "could not create Tcl interpreter" << endl;

}

const char* tcl_interp::operator () (const string& cmd) {
	Tcl_Obj* script = Tcl_NewStringObj (cmd.c_str(), -1);
	Tcl_IncrRefCount (script);
	result = Tcl_EvalObjEx (interp, script, 0);
	Tcl_DecrRefCount (script);
	return Tcl_GetStringResult (interp);
}

tcl_interp::~tcl_interp () {
	if (interp) Tcl_DeleteInterp (interp);
}
