#include "ui.h"
#include "din.h"
#include "morse_code.h"
#include "console.h"
#include "curve_editor.h"
#include "tcl_interp.h"
#include "command.h"

extern cmdlist cmdlst;
extern din din0;
extern morse_code mc;
extern console cons;
extern curve_editor delayed;
extern string dotdin;
extern tcl_interp interpreter;

const int uis_t::key [] = {SDLK_1, SDLK_2, SDLK_3, SDLK_4, SDLK_5, SDLK_6, SDLK_7, SDLK_8};
ui* uis_t::ed [] = {0};

uis_t::uis_t () {

	current = 0;
	crved = 0;
	
	uis.push_back (&din0);
	uis.push_back (&din0.stred);
	uis.push_back (&din0.waved);
	uis.push_back (&din0.chaned);
	uis.push_back (&din0.moded);
	uis.push_back (&din0.gated);
	uis.push_back (&delayed);
	uis.push_back (&din0.octed);
	uis.push_back (&din0.droneed);
	uis.push_back (&mc.ed);

}

void uis_t::set_current (ui* u, int ised) {
	if (current) current->leave ();
		current = u;
	if (ised) crved = dynamic_cast<curve_editor*>(u); else crved = 0;
	current->enter ();
	cons << console::yellow << "@ " << current->name << eol;
}

bool uis_t::handle_input () {

	cons.handle_input ();

	for (int i = 0, j = uis.size(); i < j; ++i) uis[i]->bg (); // bg processing
	
	if (cons.command_mode == 0) {
		
		current->handle_input ();
		
		static int all = 0;
		if (keypressed (SDLK_F2)) {
			if (--din0.cur_resonator < 0) din0.cur_resonator = din0.num_resonators - 1;
			hit_t hit (&din0.resonators[din0.cur_resonator].strength, din0.cur_resonator);
			din0.stred.pick (hit.crv_id);
			din0.stred.enable (hit);
			cons << console::green << "active resonator: " << din0.cur_resonator << eol;
			all = 1;
		} else 	
		if (keypressed (SDLK_F3)) {
			if (++din0.cur_resonator >= din0.num_resonators) din0.cur_resonator = 0;
			hit_t hit (&din0.resonators[din0.cur_resonator].strength, din0.cur_resonator);
			din0.stred.pick (hit.crv_id);
			din0.stred.enable (hit);
			cons << console::green << "active resonator: " << din0.cur_resonator << eol;
			all = 1;
		} else 	
		if (keypressed (SDLK_F4)) {
			if (all) {
				din0.stred.enable_all ();
				all = 0;
				cons << console::green << "all resonators enabled." << eol;
			} else {
				hit_t hit (&din0.resonators[din0.cur_resonator].strength, din0.cur_resonator);
				din0.stred.pick (hit.crv_id);
				din0.stred.enable (hit);
				cons << console::yellow << "active resonator " << din0.cur_resonator << eol;
				all = 1;
			}
			
		}
		
		for (int i = 0; i < MAX_EDITORS; ++i) {
			if (keypressed (key[i])) {
				if (ed[i]) set_current (ed[i]);
				break;
			}
		}
		
		/*if (keypressed ('7')) set_current (&fed);
		if (keypressed ('8')) { cons << console::red << "abcdefghijklmnopqrstuvwxyz ABCDEFGHIJKLMNOPQRSTUVWXYZ 1234567890 <>:;,'`#[]%+-*()" << eol; cons << console::yellow << "the quick brown fox jumps over the lazy dog" << eol;set_current (&fed);}*/
				
	}
		
	if (keypressed (SDLK_ESCAPE)) {
		if (cons.command_mode || current != &din0) {
			set_current (&din0); 
			cons.command_mode = 0;
		} else if (keydown (SDLK_LALT)) {
			interpreter ("source " + dotdin + "save_settings.tcl");
			interpreter ("quit");
		} else {
			cons << console::red << "If you want to quit, press LALT + Escape " << eol; 
			cons << console::red << "or enter quit or exit @ the command line." << eol;
		}
	} 
	return true;
}

bool uis_t::set_editor (const string& name, int screen) {
	
	if (screen > 0) {
		
		ui* ei = 0;
		for (int i = 0, j = uis.size(); i < j; ++i) {
			if (uis[i]->name == name) {
				ei = uis[i];
				break;
			}
		}
		if (ei) {
			ed [screen - 1] = ei;
			return true;
		} else {
			cmdlst.result = "bad editor name";
			return false;
		}
	} else {
		cmdlst.result = "bad screen number (valid: 1 to 8)";
		return true;
	}
}

void uis_t::draw () {
	glClear (GL_COLOR_BUFFER_BIT);
		current->draw ();
		cons.draw ();
	SDL_GL_SwapBuffers ();
}
